package com.unis.crk.business.kcgl.controller;

import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unis.crk.aop.SystemLog;
import com.unis.crk.business.kcgl.entity.DataKcglQckcDefault;
import com.unis.crk.business.kcgl.entity.vo.DataKcglQckcDefaultVo;
import com.unis.crk.business.kcgl.service.IDataKcglQckcDefaultService;
import com.unis.crk.utils.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Map;


/**
 * <p>
 * 库存管理-期初库存 前端控制器
 * </p>
 *
 * @author TangSong
 * @since 2021-12-21
 */
@CrossOrigin
@Api(value = "库存管理-期初库存",tags = {"库存管理-期初库存"})
@RestController
@RequestMapping("dataKcglQckcDefault")
public class DataKcglQckcDefaultController {

    private final static Logger LOGGER = LoggerFactory.getLogger(DataKcglQckcDefaultController.class);


    @Autowired
    private IDataKcglQckcDefaultService dataKcglQckcDefaultService;


    /**
     * 根据ID查询数据
     * @param id
     * @return
     */
    @ApiOperation(value="查询", notes="根据ID查询数据")
    @GetMapping("getOne/{id}")
    public ResultUtil getOne(@PathVariable("id") String id){
        LOGGER.info("入参为:{}",id);
//        ResultUtil<DataKcglQckcDefault> data = new ResultUtil<>();
        ResultUtil<Map<String,Object>> data = new ResultUtil<>();
        if (StringUtils.isEmpty(id)){
            data.failed("传入的主键为空");
            return data;
        }
        data.success(this.dataKcglQckcDefaultService.getOneQckc(id));
//        data.success(this.dataKcglQckcDefaultService.getById(id));
        return data;
    }

    /**
         * 分页高级查询
         * @return
         */
    @ApiOperation(value="查询", notes="分页高级查询")
    @PostMapping(value="/findPage")
    public ResultUtil findPage(@RequestBody DataKcglQckcDefaultVo pageQuery) {
        ResultUtil<IPage<Map<String,Object>>> data = new ResultUtil<>();
        data.success(this.dataKcglQckcDefaultService.findPageVo(pageQuery));
        return data;
    }
    /**
     * 新增一条数据
     * @param qckc
     * @return
     */
    @ApiOperation(value="新增", notes="新增一条数据")
    @PostMapping("add")
@SystemLog(module = "原粮期初", methods = "新增",uploadName = "yuliang.qc")
    public ResultUtil add(@RequestBody DataKcglQckcDefault qckc){
        LOGGER.info("入参为:{}", qckc);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (ObjectUtils.isEmpty(qckc)){
            data.failed("所接收对象为null");
            return data;
        }
        JSONObject ret = new JSONObject();
        try{
            ret  = this.dataKcglQckcDefaultService.saveQckc(qckc);
        }catch (Exception e){
            ret.set("success", false);
            ret.set("msg", e.getMessage());
        }
        if (ret.getBool("success")){
            data.success(ret.getStr("msg"));
        }else {
            data.failed(ret.getStr("msg"));
        }
        return data;
    }


    /**
     * 修改一条数据
     * @param qckc
     * @return
     */
    @ApiOperation(value="修改", notes="修改一条数据")
    @PatchMapping("edit")
    @SystemLog(module = "原粮期初", methods = "修改",uploadName = "yuliang.qc")
    public ResultUtil edit(@RequestBody  DataKcglQckcDefault qckc){
        LOGGER.info("入参为:{}", qckc);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (ObjectUtils.isEmpty(qckc)){
        data.failed("所接收对象为null");
        return data;
        }
        JSONObject ret = new JSONObject();
        try{
            ret  = this.dataKcglQckcDefaultService.update(qckc);
        }catch (Exception e){
            e.printStackTrace();
            ret.set("success", false);
            ret.set("msg", e.getMessage());
        }
        if (ret.getBool("success")){
            data.success(ret.getStr("msg"));
        }else {
            data.failed(ret.getStr("msg"));
        }
        return data;
    }


    /**
     * 根据ID删除数据
     * @param id
     * @return
     */
    @ApiOperation(value="删除", notes="根据ID删除数据")
    @DeleteMapping("delete/{id}")
    @SystemLog(module = "原粮期初", methods = "删除",uploadName = "yuliang.qc")
    public ResultUtil delete(@PathVariable("id") String id){
        LOGGER.info("入参为:{}", id);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (StringUtils.isEmpty(id)){
            data.failed("所接收id为null");
            return data;
        }
        JSONObject ret = new JSONObject();
        try{
            ret  = this.dataKcglQckcDefaultService.delete(id);
        }catch (Exception e){
            e.printStackTrace();
            ret.set("success", false);
            ret.set("msg", e.getMessage());
        }
        if (ret.getBool("success")){
            data.success(ret.getStr("msg"));
        }else {
            data.failed(ret.getStr("msg"));
        }
        return data;
    }


}

