package com.unis.crk.business.qcck.service.impl;

import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unis.crk.aop.SystemCountryLog;
import com.unis.crk.aop.SystemLog;
import com.unis.crk.business.qcck.entity.DataCrkywCkcmDefault;
import com.unis.crk.business.qcck.entity.vo.DataCrkywCkcmDefaultVo;
import com.unis.crk.business.qcck.entity.vo.Qcck;
import com.unis.crk.business.qcck.mapper.DataCrkywCkcmDefaultMapper;
import com.unis.crk.business.qcck.service.IDataCrkywCkcmDefaultService;
import com.unis.crk.business.qcck.service.IDataCrkywCkcmzDefaultService;
import com.unis.crk.business.qcck.service.IDataCrkywCkcpzDefaultService;
import com.unis.crk.business.qcck.service.IDataCrkywCkrmdjDefaultService;
import com.unis.crk.common.entity.Bizflowtrail;
import com.unis.crk.common.service.IBizflowtrailService;
import com.unis.crk.common.service.IDeleteDataRecordService;
import com.unis.crk.utils.ResultUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 汽车出库-出门表 服务实现类
 * </p>
 *
 * @author TangSong
 * @since 2021-10-22
 */
@Service
public class DataCrkywCkcmDefaultServiceImpl extends ServiceImpl<DataCrkywCkcmDefaultMapper, DataCrkywCkcmDefault> implements IDataCrkywCkcmDefaultService {

    @Resource
    private DataCrkywCkcmDefaultMapper dataCrkywCkcmDefaultMapper;
    @Autowired
    private IBizflowtrailService bizflowtrailService;
    @Autowired
    private IDeleteDataRecordService deleteDataRecordService;

    @Autowired
    IDataCrkywCkrmdjDefaultService dataCrkywCkrmdjDefaultService;
    @Autowired
    IDataCrkywCkcpzDefaultService dataCrkywCkcpzDefaultService;
    @Autowired
    IDataCrkywCkcmzDefaultService dataCrkywCkcmzDefaultService;
    /**
     * 通过单位号检查是否已经存在
     *
     * @param bizNo
     * @return
     */
    private boolean isExist(String bizNo) {
        QueryWrapper<DataCrkywCkcmDefault> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("BizNo", bizNo);
        queryWrapper.last("limit 1");
        return super.getOne(queryWrapper) != null;
    }
    /**
     * 保存汽车出库-出门表
     *
     * @param qcck 汽车出库-出门表业务数据
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
//    @SystemLog(module = "原粮出库出门", methods = "新增",uploadName = "yuliang.ck")
    @SystemCountryLog(module = "原粮出库出门", methods = "新增",uploadName = "yuliang.ck")
    public JSONObject save(Qcck qcck) throws Exception {
        JSONObject jsonObject = new JSONObject();
        jsonObject.set("success", false);
        DataCrkywCkcmDefault entity = qcck.getDataCrkywCkcmDefault();
        if (isExist(entity.getBizNo())){
            jsonObject.set("msg", "此单据号已存在");
            return jsonObject;
        }
        entity.setSytelInfoID("4028814763a546b20163aea8547e0027"); //样式id
        entity.setPeriodID("11"); //期间id
        entity.setDeptID(entity.getUnitID());
        entity.setRoundID(0); //舍位id
        entity.setShoworder(0);
        entity.setPreorder(3);
        if (StringUtils.isBlank(entity.getBuilder())){
            entity.setBuilder(entity.getUnitID());
            entity.setEditor(entity.getUnitID());
        }
        if (!super.save(entity)) { //保存汽车出库-出门表表成功
            jsonObject.set("msg", "保存汽车出库-出门表失败");
            return jsonObject;
        }

        if (!saveBizflowtrailEnt(qcck)) { //保存轨迹表
            jsonObject.set("msg", "保存轨迹表失败");
            throw new Exception("保存轨迹表失败");
//        return  jsonObject;
        }
        jsonObject.set("success", true);
        //为上报国家局
        jsonObject.set("dataId", entity.getDataID());
        jsonObject.set("bizNo", entity.getBizNo());
        jsonObject.set("msg", "保存成功");
        return jsonObject;
    }
    /**
     * 保存汽车出库数据
     *
     * @param qcck 汽车出库业务数据
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public JSONObject addQcck(Qcck qcck) throws Exception {
        JSONObject jsonObject = new JSONObject();
        //保存入门数据
        jsonObject = dataCrkywCkrmdjDefaultService.save(qcck);
        if (!jsonObject.getBool("success")){
            return jsonObject;
        }
        //保存称皮重数据
        jsonObject = dataCrkywCkcpzDefaultService.save(qcck);
        if (!jsonObject.getBool("success")){
            return jsonObject;
        }
        //保存称毛重数据
        jsonObject = dataCrkywCkcmzDefaultService.save(qcck);
        if (!jsonObject.getBool("success")){
            return jsonObject;
        }
        //保存出门数据
        jsonObject = this.save(qcck);
        if (!jsonObject.getBool("success")){
            return jsonObject;
        }
        jsonObject.set("success", true);
        jsonObject.set("msg", "保存成功");
        return jsonObject;
    }

    /**
     * 修改汽车出库-出门表
     *
     * @param qcrk 汽车出库-出门表业务数据
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
//    @SystemLog(module = "原粮出库出门", methods = "修改",uploadName = "yuliang.ck")
    @SystemCountryLog(module = "原粮出库出门", methods = "修改",uploadName = "yuliang.ck")
    public JSONObject update(Qcck qcrk) throws Exception {
        {
            {
                JSONObject jsonObject = new JSONObject();
                jsonObject.set("success", false);
                //查询下一环节是否已处理
                DataCrkywCkcmDefault entity = qcrk.getDataCrkywCkcmDefault();
                String bizNo = qcrk.getBizflowtrail().getBillNo();
                String unitId = entity.getUnitID();
                Integer curBizNodeID = 3; //当前节点
                //条件查询器
                QueryWrapper<Bizflowtrail> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("BillNo", bizNo);
                queryWrapper.eq("UnitID", unitId);
                queryWrapper.eq("CurBizNodeID", curBizNodeID); //当前节点
                queryWrapper.last("limit 1");
                Bizflowtrail bizflowtrail = bizflowtrailService.getOne(queryWrapper);
                if (bizflowtrail == null) {
                    jsonObject.set("msg", "轨迹表数据已被删除，请联系管理员");
                    return jsonObject;
                }

                if (!super.updateById(entity)) { //修改汽车出库-出门表表成功
                    jsonObject.set("msg", "保存汽车出库-出门表失败");
                    throw new Exception("保存汽车出库");
//        return  jsonObject;
                }
                jsonObject.set("success", true);
                //为上报国家局
                jsonObject.set("dataId", entity.getDataID());
                jsonObject.set("bizNo", bizNo);
                jsonObject.set("msg", "保存成功");
                return jsonObject;
            }
        }
    }

    /**
     * 删除汽车出库-出门表数据
     *
     * @param id 汽车出库-出门表主键尖
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
//    @SystemLog(module = "原粮出库出门", methods = "删除",uploadName = "yuliang.ck")
    @SystemCountryLog(module = "原粮出库出门", methods = "删除",uploadName = "yuliang.ck")
    public JSONObject delete(String id) throws Exception {
        {
            JSONObject jsonObject = new JSONObject();
            jsonObject.set("success", false);
            //得到对象
            DataCrkywCkcmDefault entity = super.getById(id);
            if (entity == null) {
                jsonObject.set("msg", "此数据已被删除");
                return jsonObject;
            }
            String bizNo = entity.getBizNo();
            String unitId = entity.getUnitID();
            Integer curBizNodeID = 3; //当前节点
            //条件查询器
            QueryWrapper<Bizflowtrail> deleteWrapper = new QueryWrapper<>();
            deleteWrapper.eq("BillNo", bizNo);
            deleteWrapper.eq("UnitID", unitId);
            deleteWrapper.eq("CurBizNodeID", curBizNodeID); //当前节点
            deleteWrapper.last("limit 1");
            Bizflowtrail bizflowtrail = bizflowtrailService.getOne(deleteWrapper);
            if (bizflowtrail == null) {

                jsonObject.set("msg", "轨迹表数据已被删除，请联系管理员");
                return jsonObject;
            }
            if (!super.removeById(id)) { //删除汽车扦样表成功
                jsonObject.set("msg", "删除汽车出库-出门表数据失败");
                return jsonObject;
            }
            //修改当前节点状态为0
            UpdateWrapper<Bizflowtrail> updateWrapper = new UpdateWrapper<>();
            updateWrapper.eq("BillNo", bizNo);
            updateWrapper.eq("UnitID", unitId);
            updateWrapper.eq("CurBizNodeID", curBizNodeID);
            updateWrapper.set("status", 0);
            updateWrapper.set("UPDATE_TIME", null);
            if (!bizflowtrailService.update(updateWrapper)) {
                jsonObject.set("msg", "修改当前轨迹数据表失败");
                throw new Exception("修改当前轨迹数据表失败");
//        return  jsonObject;
            }
            //保存删除数据记录
            deleteDataRecordService.save(id,"data_crkyw_ckcm_default",entity.getUnitID(),new Date());

            jsonObject.set("success", true);
            jsonObject.set("dataId", entity.getDataID());
            jsonObject.set("bizNo", entity.getBizNo());
            jsonObject.set("msg", "删除成功");
            return jsonObject;
        }
    }

    /**
     * 按照条件查询汽车出库-出门表信息
     *
     * @param pageQuery
     * @throws Exception
     */
    @Override
    public IPage<Map<String, Object>> findPageVo(DataCrkywCkcmDefaultVo pageQuery) {
            //条形查询器
            QueryWrapper<DataCrkywCkcmDefault> queryWrapper = new QueryWrapper<>();
            if (StringUtils.isNotBlank(pageQuery.getUnitId())) {
                queryWrapper.eq("DATA_CRKYW_CKCM_DEFAULT.UnitId", pageQuery.getUnitId());
            }
            if (StringUtils.isNotBlank(pageQuery.getBizNo())) {
                queryWrapper.like("DATA_CRKYW_CKCM_DEFAULT.bizNo", ResultUtil.getQueryWrapperformat(pageQuery.getBizNo()));
            }
            if (StringUtils.isNotBlank(pageQuery.getLspz())) {
                queryWrapper.eq("DATA_CRKYW_CKRMDJ_DEFAULT.lspz", pageQuery.getLspz());
            }
            if (StringUtils.isNotBlank(pageQuery.getXz())) {
                queryWrapper.eq("DATA_CRKYW_CKRMDJ_DEFAULT.xz", pageQuery.getXz());
            }
            if (StringUtils.isNotBlank(pageQuery.getSlr())) {
                queryWrapper.like("DATA_CRKYW_CKRMDJ_DEFAULT.slr", ResultUtil.getQueryWrapperformat(pageQuery.getSlr()));
            }
            if (StringUtils.isNotBlank(pageQuery.getCyr())) {
                queryWrapper.like("DATA_CRKYW_CKRMDJ_DEFAULT.cyr", ResultUtil.getQueryWrapperformat(pageQuery.getCyr()));
            }
            if (StringUtils.isNotBlank(pageQuery.getCydw())){
                queryWrapper.like("DATA_CRKYW_CKRMDJ_DEFAULT.cydw",ResultUtil.getQueryWrapperformat(pageQuery.getCydw()));
            }
            if (StringUtils.isNotBlank(pageQuery.getCph())) {
                queryWrapper.like("DATA_CRKYW_CKRMDJ_DEFAULT.cph", ResultUtil.getQueryWrapperformat(pageQuery.getCph()));
            }
            if (StringUtils.isNotBlank(pageQuery.getCmsjStart()) && StringUtils.isNotBlank(pageQuery.getCmsjEnd())) {
                QueryWrapper<DataCrkywCkcmDefault> apply = queryWrapper.apply(" DATE_FORMAT(ifnull(DATA_CRKYW_CKCM_DEFAULT.cmsj,NOW()),'%Y-%m-%d') >=  '" + pageQuery.getCmsjStart() + "' AND DATE_FORMAT(ifnull(DATA_CRKYW_CKCM_DEFAULT.cmsj,NOW()),'%Y-%m-%d') <= '" + pageQuery.getCmsjEnd() + " 23:59:59'");
            }
            //排序
            queryWrapper.orderByDesc("DATA_CRKYW_CKCM_DEFAULT.BuildDate");

            //分页相关
            Page<DataCrkywCkcmDefault> p = new Page<>(pageQuery.getPage(), pageQuery.getLimit());

            IPage<Map<String, Object>> page = dataCrkywCkcmDefaultMapper.selectMyMapsPage(p, queryWrapper);
            return page;
    }

    @Override
    public List<Map<String,Object>> getPerData(DataCrkywCkcmDefaultVo pageQuery) {
        List<Map<String,Object>> list = dataCrkywCkcmDefaultMapper.getPerData(pageQuery);
        return list;
    }

    @Override
    public Map<String, Object> getOne(String bizNo, String id) {
        return dataCrkywCkcmDefaultMapper.getOne(bizNo,id);
    }

    private Boolean saveBizflowtrailEnt(Qcck qcrk) {
        DataCrkywCkcmDefault dataCrkywRkcmDefault = qcrk.getDataCrkywCkcmDefault();
        String bizNo = dataCrkywRkcmDefault.getBizNo(); //单据号
        String unitId = dataCrkywRkcmDefault.getUnitID(); //单位ID
        Integer curBizNodeID = 3; //当前节点
        Integer nextBizNodeID = 4; //下一节点
        Integer bizNodeId = 13; //  report表节点id
        Integer trailOrder = 3;
        //更新上一环节业务数据
        //更新查询器
        UpdateWrapper<Bizflowtrail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("BillNo", bizNo);
        updateWrapper.eq("UnitID", unitId);
        updateWrapper.eq("CurBizNodeID", curBizNodeID);
        updateWrapper.eq("NextBizNodeID", nextBizNodeID);
        updateWrapper.eq("BizNodeId", bizNodeId);
        updateWrapper.eq("TrailOrder", trailOrder);
        updateWrapper.set("status", 100); //修改状态，完成
        if (!bizflowtrailService.update(updateWrapper)) { //更新轨迹表
            return false;
        }
        return true;
    }
}
