package com.unis.crk.business.qcrk.service.impl;

import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unis.crk.aop.SystemLog;
import com.unis.crk.business.qcrk.entity.DataCrkywJsxxDefault;
import com.unis.crk.business.qcrk.entity.vo.DataCrkywCmzDefaultVo;
import com.unis.crk.business.qcrk.entity.vo.DataCrkywJsxxDefaultVo;
import com.unis.crk.business.qcrk.entity.vo.Qcrk;
import com.unis.crk.business.qcrk.mapper.DataCrkywJsxxDefaultMapper;
import com.unis.crk.business.qcrk.service.IDataCrkywJsxxDefaultService;
import com.unis.crk.business.qcrk.service.IDataCrkywJyxxFlt001Service;
import com.unis.crk.common.entity.Bizflowtrail;
import com.unis.crk.common.service.IBizflowtrailService;
import com.unis.crk.common.service.IDeleteDataRecordService;
import com.unis.crk.utils.ResultUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 汽车入库-结算主表 服务实现类
 * </p>
 *
 * @author TangSong
 * @since 2021-10-11
 */
@Service
public class DataCrkywJsxxDefaultServiceImpl extends ServiceImpl<DataCrkywJsxxDefaultMapper, DataCrkywJsxxDefault> implements IDataCrkywJsxxDefaultService {

    @Resource
    private DataCrkywJsxxDefaultMapper dataCrkywJsxxDefaultMapper;
    @Autowired
    private IDataCrkywJyxxFlt001Service dataCrkywJyxxFlt001Service;
    @Autowired
    private IBizflowtrailService bizflowtrailService;
    @Autowired
    private IDeleteDataRecordService deleteDataRecordService;
    /**
     * 通过单位号检查是否已经存在
     *
     * @param bizNo
     * @return
     */
    private boolean isExist(String bizNo) {
        QueryWrapper<DataCrkywJsxxDefault> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("BizNo", bizNo);
        queryWrapper.last("limit 1");
        return super.getOne(queryWrapper) != null;
    }
    /**
     * 保存汽车入库-结算主表
     *
     * @param qcrk 汽车入库-结算主表业务数据
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @SystemLog(module = "原粮入库新增结算", methods = "新增",uploadName = "yuliang.rk")
    public JSONObject save(Qcrk qcrk)throws  Exception {
        JSONObject jsonObject = new JSONObject();
        jsonObject.set("success", false);
        DataCrkywJsxxDefault entity = qcrk.getDataCrkywJsxxDefault();
        if (isExist(entity.getBizNo())){
            jsonObject.set("msg","此单据号已存在");
            return jsonObject;
        }
        entity.setSytelInfoID("4028814763a546b20163aa64378d0008"); //样式id
        entity.setPeriodID("11"); //期间id
        entity.setDeptID(entity.getUnitID());//部门id
        entity.setRoundID(0); //舍们id
        entity.setShoworder(0);
        entity.setPreorder(5);
        if (StringUtils.isBlank(entity.getBuilder())){
            entity.setBuilder(entity.getUnitID());
            entity.setEditor(entity.getUnitID());
        }
        boolean b = super.saveOrUpdate(entity);
        if (!b) { ///保存汽车入库-结算主表表成功
            jsonObject.set("msg","保存汽车入库-结算主表失败");
            return  jsonObject;

        }
        if (!saveBizflowtrailEnt(qcrk)) { //保存轨迹表
            jsonObject.set("msg", "保存轨迹表失败");
            throw  new Exception("保存轨迹表失败");
//            return jsonObject;
        }
        jsonObject.set("success", true);
        jsonObject.set("msg", "保存成功");
        return jsonObject;
    }

    /**
     * 修改汽车入库-结算主表
     *
     * @param qcrk 汽车入库-结算主表业务数据
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @SystemLog(module = "原粮入库修改结算", methods = "修改",uploadName = "yuliang.rk")
    public JSONObject update(Qcrk qcrk)throws  Exception {
        {
            {
                JSONObject jsonObject = new JSONObject();
                jsonObject.set("success", false);
                //查询下一环节是否已处理
                DataCrkywJsxxDefault entity = qcrk.getDataCrkywJsxxDefault();
                String bizNo = entity.getBizNo();
                String unitId = entity.getUnitID();
                Integer curBizNodeID = 6; //当前节点
                //条件查询器
                QueryWrapper<Bizflowtrail> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("BillNo", bizNo);
                queryWrapper.eq("UnitID", unitId);
                queryWrapper.eq("CurBizNodeID", curBizNodeID); //当前节点
                queryWrapper.last("limit 1");
                Bizflowtrail bizflowtrail = bizflowtrailService.getOne(queryWrapper);
                if (bizflowtrail == null) {
                    jsonObject.set("msg", "轨迹表数据已被删除，请联系管理员");
                    return jsonObject;
                }
                if (bizflowtrail.getStatus() == 1 || bizflowtrail.getStatus() == 100) { //表示已经执行后面业务，不能删除
                    jsonObject.set("msg", "当前车辆已完成[汽车入库-结算信息]，不能修改");
                    return jsonObject;
                }
                if (!super.updateById(entity)) { //修改汽车入库-结算主表表成功
                    jsonObject.set("msg", "修改汽汽车入库-结算信息失败");
                    return jsonObject;
                }

                jsonObject.set("success", true);
                jsonObject.set("msg", "保存成功");
                return jsonObject;
            }
        }
    }

    /**
     * 删除汽车入库-结算主表数据
     *
     * @param id 汽车入库-结算主表主键尖
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @SystemLog(module = "原粮入库删除结算", methods = "删除",uploadName = "yuliang.rk")
    public JSONObject delete(String id) throws  Exception{
        {
            JSONObject jsonObject = new JSONObject();
            jsonObject.set("success", false);
            //得到对象
            DataCrkywJsxxDefault entity = super.getById(id);
            if (entity == null) {
                jsonObject.set("msg", "此数据已被删除");
                return jsonObject;
            }
            String bizNo = entity.getBizNo();
            String unitId = entity.getUnitID();
            Integer curBizNodeID = 6; //当前节点
            //条件查询器
            QueryWrapper<Bizflowtrail> deleteWrapper = new QueryWrapper<>();
            deleteWrapper.eq("BillNo", bizNo);
            deleteWrapper.eq("UnitID", unitId);
            deleteWrapper.eq("CurBizNodeID", curBizNodeID); //当前节点
            deleteWrapper.last("limit 1");
            Bizflowtrail bizflowtrail = bizflowtrailService.getOne(deleteWrapper);
            if (bizflowtrail == null) {
                jsonObject.set("msg", "轨迹表数据已被删除，请联系管理员");
                return jsonObject;
            }
            if (bizflowtrail.getStatus() == 1 || bizflowtrail.getStatus() == 100) { //表示已经执行后面业务，不能删除
                jsonObject.set("msg", "当前车辆已完成[汽车入库-结算主表]，不能删除");
                return jsonObject;
            }
            if (!super.removeById(id)) { //删除汽车扦样表成功
                jsonObject.set("msg", "删除汽车入库-结算信息数据失败");
                return jsonObject;
            }
            //条件查询器
            if (!bizflowtrailService.remove(deleteWrapper)) { //删除轨迹表,当前节点数据
                jsonObject.set("msg", "删除轨迹表数据失败");
                throw  new Exception("删除轨迹表数据失败");
//                return jsonObject;
            }
            //修改上一个节点状态为0
            UpdateWrapper<Bizflowtrail> updateWrapper = new UpdateWrapper<>();
            updateWrapper.eq("BillNo", bizNo);
            updateWrapper.eq("UnitID", unitId);
            updateWrapper.eq("CurBizNodeID", curBizNodeID - 1);
            updateWrapper.set("status", 0);
            updateWrapper.set("UPDATE_TIME", null);
            if (!bizflowtrailService.update(updateWrapper)) {
                jsonObject.set("msg", "修改上一节点轨迹数据表失败");
                throw  new Exception("修改上一节点轨迹数据表失败");
//                return jsonObject;
            }
            //保存删除数据记录
            deleteDataRecordService.save(id,"data_crkyw_jsxx_default",unitId,new Date());
            deleteDataRecordService.save(bizflowtrail.getTrailID(),"bizflowtrail",unitId,new Date());

            jsonObject.set("success", true);
            jsonObject.set("msg", "删除成功");
            return jsonObject;
        }
    }

    /**
     * 按照条件查询汽车入库-结算主表信息
     *
     * @param pageQuery
     * @throws Exception
     */
    @Override
    public IPage<Map<String, Object>> findPageVo(DataCrkywJsxxDefaultVo pageQuery) {
            //条形查询器
            QueryWrapper<DataCrkywJsxxDefault> queryWrapper = new QueryWrapper<>();
            if (StringUtils.isNotBlank(pageQuery.getUnitId())){
                queryWrapper.eq("DATA_CRKYW_JSXX_DEFAULT.UnitId",pageQuery.getUnitId());
            }
            if (StringUtils.isNotBlank(pageQuery.getBizNo())){
                queryWrapper.like("DATA_CRKYW_JSXX_DEFAULT.bizNo", ResultUtil.getQueryWrapperformat(pageQuery.getBizNo()));
            }
            if (StringUtils.isNotBlank(pageQuery.getSlr())){
                queryWrapper.like("DATA_CRKYW_RMDJ_DEFAULT.slr",ResultUtil.getQueryWrapperformat(pageQuery.getSlr()));
            }
            if (StringUtils.isNotBlank(pageQuery.getCyr())){
                queryWrapper.like("DATA_CRKYW_RMDJ_DEFAULT.cyr",ResultUtil.getQueryWrapperformat(pageQuery.getCyr()));
            }
            if (StringUtils.isNotBlank(pageQuery.getCydw())){
                queryWrapper.like("DATA_CRKYW_RMDJ_DEFAULT.cydw",ResultUtil.getQueryWrapperformat(pageQuery.getCydw()));
            }
            if (StringUtils.isNotBlank(pageQuery.getLspz())){
                queryWrapper.eq("DATA_CRKYW_RMDJ_DEFAULT.lspz",ResultUtil.getQueryWrapperformat(pageQuery.getLspz()));
            }
            if (StringUtils.isNotBlank(pageQuery.getCph())){
                queryWrapper.like("DATA_CRKYW_RMDJ_DEFAULT.cph",ResultUtil.getQueryWrapperformat(pageQuery.getCph()));
            }
            if (StringUtils.isNotBlank(pageQuery.getJssjStart()) && StringUtils.isNotBlank(pageQuery.getJssjEnd())){
                queryWrapper.apply(" DATE_FORMAT(ifnull(DATA_CRKYW_JSXX_DEFAULT.jssj,NOW()),'%Y-%m-%d') >=  '"+pageQuery.getJssjStart()+"' AND DATE_FORMAT(ifnull(DATA_CRKYW_JSXX_DEFAULT.jssj,NOW()),'%Y-%m-%d') <= '"+pageQuery.getJssjEnd()+" 23:59:59'");

            }
            //排序
            queryWrapper.orderByDesc("DATA_CRKYW_JSXX_DEFAULT.BuildDate");

            //分页相关
            Page<DataCrkywJsxxDefault> p = new Page<>(pageQuery.getPage(), pageQuery.getLimit());

            IPage<Map<String, Object>> page = dataCrkywJsxxDefaultMapper.selectMyMapsPage(p, queryWrapper);
            return page;
    }
    @Override
    public DataCrkywJsxxDefault getAllById(Serializable id) {
        DataCrkywJsxxDefault dataCrkywJsxxDefault = super.getById(id);
        return dataCrkywJsxxDefault;
    }

    @Override
    public List<Map<String, Object>> getPerData(DataCrkywCmzDefaultVo pageQuery) {
        List<Map<String, Object>> list = dataCrkywJsxxDefaultMapper.getPerData(pageQuery);
        return list;
    }

    @Override
    public Map<String, Object> getOne(String bizNo, String id) {
        Map<String,Object> map = dataCrkywJsxxDefaultMapper.getOne(bizNo,id);
        //得到检验信息
        List<Map<String,Object>> fltList1 = dataCrkywJyxxFlt001Service.getLitByBizNoOrMainDatId(bizNo,null);
        map.put("fltList1",fltList1);
        return map;
    }

    private Boolean saveBizflowtrailEnt(Qcrk qcrk) {
        Bizflowtrail bizflowtrail = new Bizflowtrail();
        DataCrkywJsxxDefault dataCrkywJsxxDefault = qcrk.getDataCrkywJsxxDefault();
        String flowId = qcrk.getBizflowtrail().getFlowID(); //流程id
        String bizNo = dataCrkywJsxxDefault.getBizNo(); //单据号
        String unitId = dataCrkywJsxxDefault.getUnitID(); //单位ID
        Integer curBizNodeID = 6; //当前节点
        Integer nextBizNodeID = 7; //下一节点
        Integer bizNodeId = 7; //  report表节点id
        Integer trailOrder = 6;
        String builder = dataCrkywJsxxDefault.getBuilder(); //创建人
        Integer showOrder = dataCrkywJsxxDefault.getShoworder(); //序号

        bizflowtrail.setBillNo(bizNo);
        bizflowtrail.setUnitID(unitId);
        bizflowtrail.setTrailOrder(trailOrder);
        bizflowtrail.setCurBizNodeID(curBizNodeID);
        bizflowtrail.setNextBizNodeID(nextBizNodeID);
        bizflowtrail.setBizNodeId(bizNodeId);
        bizflowtrail.setBuilder(builder);
        bizflowtrail.setBuildDate(new Date());
        bizflowtrail.setShowOrder(showOrder);
        bizflowtrail.setFlowID(flowId);
        bizflowtrail.setStatus(0);
        if (!bizflowtrailService.save(bizflowtrail)) {
            return false;
        }
        //更新上一环节业务数据
        //更新查询器
        UpdateWrapper<Bizflowtrail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("BillNo", bizNo);
        updateWrapper.eq("UnitID", unitId);
        updateWrapper.eq("CurBizNodeID", curBizNodeID - 1);
        updateWrapper.eq("NextBizNodeID", nextBizNodeID - 1);
        updateWrapper.eq("BizNodeId", bizNodeId - 1);
        updateWrapper.eq("TrailOrder", trailOrder - 1);
        updateWrapper.set("status", 1); //修改状态
        if (!bizflowtrailService.update(updateWrapper)) { //更新轨迹表
            return false;
        }
        return true;
    }
}
