package com.unis.crk.business.qcrk.service.impl;

import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unis.crk.aop.SystemCountryLog;
import com.unis.crk.aop.SystemLog;
import com.unis.crk.business.qcrk.entity.DataCrkywRmdjDefault;
import com.unis.crk.business.qcrk.entity.vo.DataCrkywRmdjDefaultVo;
import com.unis.crk.business.qcrk.entity.vo.Qcrk;
import com.unis.crk.business.qcrk.mapper.DataCrkywRmdjDefaultMapper;
import com.unis.crk.business.qcrk.mapper.DataCrkywRmdjDefaultMapper;
import com.unis.crk.business.qcrk.service.IDataCrkywRmdjDefaultService;
import com.unis.crk.common.entity.Bizflowtrail;
import com.unis.crk.common.service.IBizflowtrailService;
import com.unis.crk.common.service.IDeleteDataRecordService;
import com.unis.crk.utils.ResultUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 汽车入库-入门登记 服务实现类
 * </p>
 *
 * @author TangSong
 * @since 2021-11-16
 */
@Service
public class DataCrkywRmdjDefaultServiceImpl extends ServiceImpl<DataCrkywRmdjDefaultMapper, DataCrkywRmdjDefault> implements IDataCrkywRmdjDefaultService {

    @Resource
    private DataCrkywRmdjDefaultMapper dataCrkywRmdjDefaultMapper;
    @Autowired
    private IBizflowtrailService bizflowtrailService;
    @Autowired
    private IDeleteDataRecordService deleteDataRecordService;
    /**
     * 通过单位号检查是否已经存在
     *
     * @param bizNo
     * @return
     */
    private boolean isExist(String bizNo) {
        QueryWrapper<DataCrkywRmdjDefault> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("BizNo", bizNo);
        queryWrapper.last("limit 0,1");
        return super.getOne(queryWrapper) != null;
    }
    @Override
    @Transactional(rollbackFor = Exception.class)
    @SystemLog(module = "原粮入库修改入门", methods = "增加",uploadName = "yuliang.rk")
    public JSONObject save(Qcrk qcrk) throws  Exception{
        JSONObject jsonObject = new JSONObject();
        jsonObject.set("success",false);
        DataCrkywRmdjDefault dataCrkywRmdjDefault = qcrk.getDataCrkywRmdjDefault();
        if (isExist(dataCrkywRmdjDefault.getBizNo())){
            jsonObject.set("msg","此单据号已存在");
            return jsonObject;
        }
        dataCrkywRmdjDefault.setSytelInfoID("402881476395411d016396360bd70003"); //样式id
        dataCrkywRmdjDefault.setPeriodID("11"); //期间id
        dataCrkywRmdjDefault.setDeptID(dataCrkywRmdjDefault.getUnitID());
        dataCrkywRmdjDefault.setRoundID(0); //舍们id
        dataCrkywRmdjDefault.setShowOrder(0);
        dataCrkywRmdjDefault.setPreOrder(0);
        if (StringUtils.isBlank(dataCrkywRmdjDefault.getBuilder())){
            dataCrkywRmdjDefault.setBuilder(dataCrkywRmdjDefault.getUnitID());
            dataCrkywRmdjDefault.setEditor(dataCrkywRmdjDefault.getUnitID());
        }
        if (!super.save(dataCrkywRmdjDefault)) { //保存汽车入门表成功
            throw  new Exception("保存汽车入门失败");
        }if (!saveBizflowtrailEnt(qcrk)) { //保存轨迹表
//            jsonObject.set("msg","保存轨迹表失败");
            throw  new Exception("保存轨迹表失败");
//            return  jsonObject;
        }
        jsonObject.set("success",true);
        jsonObject.set("msg","保存成功");
        return jsonObject;
    }
    @Override
    @Transactional(rollbackFor = Exception.class)
    @SystemLog(module = "原粮入库修改入门", methods = "修改",uploadName = "yuliang.rk")
    public  JSONObject update(Qcrk qcrk){
        JSONObject jsonObject = new JSONObject();
        jsonObject.set("success",false);
        //查询下一环节是否已处理
        DataCrkywRmdjDefault dataCrkywRmdjDefault = qcrk.getDataCrkywRmdjDefault();
        String bizNo = dataCrkywRmdjDefault.getBizNo();
        String unitId = dataCrkywRmdjDefault.getUnitID();
        Integer curBizNodeID = 1; //当前节点
        //条件查询器
        QueryWrapper<Bizflowtrail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("BillNo",bizNo);
        queryWrapper.eq("UnitID",unitId);
        queryWrapper.eq("CurBizNodeID",curBizNodeID); //当前节点
        queryWrapper.last("limit 1");
        Bizflowtrail bizflowtrail = bizflowtrailService.getOne(queryWrapper);
        if (bizflowtrail == null){
            jsonObject.set("msg","轨迹表数据已被删除，请联系管理员");
            return  jsonObject;
        }
        if (bizflowtrail.getStatus() == 1 || bizflowtrail.getStatus() == 100){ //表示已经执行后面业务，不能删除
            jsonObject.set("msg","当前车辆已完成[扦样]，不能修改");
            return  jsonObject;
        }
        if (!super.updateById(dataCrkywRmdjDefault)) { //修改汽车入门表成功
            jsonObject.set("msg","修改汽车入门失败");
            return  jsonObject;
        }
        jsonObject.set("success",true);
        jsonObject.set("msg","保存成功");
        return jsonObject;
    }
    @Override
    @Transactional(rollbackFor = Exception.class)
    @SystemLog(module = "原粮入库删除入门", methods = "删除",uploadName = "yuliang.rk")
    public JSONObject delete(String id) throws  Exception {
        JSONObject jsonObject = new JSONObject();
        jsonObject.set("success",false);

        //得到对象
        DataCrkywRmdjDefault dataCrkywRmdjDefault = super.getById(id);
        if (dataCrkywRmdjDefault == null){
            jsonObject.set("msg","此数据已被删除");
            return  jsonObject;
        }
        String bizNo = dataCrkywRmdjDefault.getBizNo();
        String unitId = dataCrkywRmdjDefault.getUnitID();
        Integer curBizNodeID = 1; //当前节点
        //条件查询器
        QueryWrapper<Bizflowtrail> deleteWrapper = new QueryWrapper<>();
        deleteWrapper.eq("BillNo",bizNo);
        deleteWrapper.eq("UnitID",unitId);
        deleteWrapper.eq("CurBizNodeID",curBizNodeID); //当前节点
        deleteWrapper.last("limit 1");
        Bizflowtrail bizflowtrail = bizflowtrailService.getOne(deleteWrapper);
        if (bizflowtrail == null){
            jsonObject.set("msg","轨迹表数据已被删除，请联系管理员");
            return  jsonObject;
        }
        if (bizflowtrail.getStatus() == 1 || bizflowtrail.getStatus() == 100){ //表示已经执行后面业务，不能删除
            jsonObject.set("msg","当前车辆已完成[扦样]，不能删除");
            return  jsonObject;
        }
        if (!super.removeById(id)) { //删除汽车入门表成功
            jsonObject.set("msg","删除汽车入门失败");
            return  jsonObject;
        }
        if (!bizflowtrailService.remove(deleteWrapper)) { //删除轨迹表
            jsonObject.set("msg","删除轨迹表失败");
            throw  new Exception("删除轨迹表失败");
//            return  jsonObject;
        }
        //保存删除数据记录
        deleteDataRecordService.save(id,"data_crkyw_rkdj_default",unitId,new Date());
        deleteDataRecordService.save(bizflowtrail.getTrailID(),"bizflowtrail",unitId,new Date());

        jsonObject.set("success",true);
        jsonObject.set("msg","删除成功");
        return jsonObject;
    }

    @Override
    public IPage<Map<String, Object>> findPageVo(DataCrkywRmdjDefaultVo pageQuery) {
        {
            //条形查询器
            QueryWrapper<DataCrkywRmdjDefault> queryWrapper = new QueryWrapper<>();
            if (StringUtils.isNotBlank(pageQuery.getUnitId())) {
                queryWrapper.eq("a.UnitId", pageQuery.getUnitId());
            }
            if (StringUtils.isNotBlank(pageQuery.getBizNo())) {
                queryWrapper.like("a.bizNo", ResultUtil.getQueryWrapperformat(pageQuery.getBizNo()));
            }
            if (StringUtils.isNotBlank(pageQuery.getLspz())) {
                queryWrapper.like("a.lspz", ResultUtil.getQueryWrapperformat(pageQuery.getLspz()));
            }
            if (StringUtils.isNotBlank(pageQuery.getCyr())) {
                queryWrapper.like("a.cyr", ResultUtil.getQueryWrapperformat(pageQuery.getCyr()));
            }
            if (StringUtils.isNotBlank(pageQuery.getCydw())) {
                queryWrapper.like("a.cydw", ResultUtil.getQueryWrapperformat(pageQuery.getCydw()));
            }
            if (StringUtils.isNotBlank(pageQuery.getCph())) {
                queryWrapper.like("a.cph", ResultUtil.getQueryWrapperformat(pageQuery.getCph()));
            }
            if (StringUtils.isNotBlank(pageQuery.getDjsjStart()) && StringUtils.isNotBlank(pageQuery.getDjsjEnd())) {
                queryWrapper.apply(" DATE_FORMAT(ifnull(a.ywdjsj,NOW()),'%Y-%m-%d') >=  '" + pageQuery.getDjsjStart() + "' AND DATE_FORMAT(ifnull(a.ywdjsj,NOW()),'%Y-%m-%d') <= '" + pageQuery.getDjsjEnd() + " 23:59:59'");
            }
            //排序
            queryWrapper.orderByDesc("a.ModifyDate");

            //分页相关
            Page<DataCrkywRmdjDefault> p = new Page<>(pageQuery.getPage(), pageQuery.getLimit());

            IPage<Map<String, Object>> page = dataCrkywRmdjDefaultMapper.selectMyMapsPage(p, queryWrapper);
            return page;
        }
    }

    @Override
    public Map<String, Object> getOne(String bizNo, String id) {
        Map<String,Object> map = dataCrkywRmdjDefaultMapper.getOne(bizNo,id);
        return map;
    }

    private boolean saveBizflowtrailEnt(Qcrk qcrk){
        Bizflowtrail bizflowtrail = new Bizflowtrail();
        DataCrkywRmdjDefault dataCrkywRmdjDefault= qcrk.getDataCrkywRmdjDefault();
        String flowId = qcrk.getBizflowtrail().getFlowID(); //流程id
        String bizNo = dataCrkywRmdjDefault.getBizNo(); //单据号
        String unitId = dataCrkywRmdjDefault.getUnitID(); //单位ID
        Integer curBizNodeID = 1; //当前节点
        Integer nextBizNodeID = 2; //下一节点
        Integer bizNodeId = 14; //  report表节点id
        Integer trailOrder = 1;
        String builder = dataCrkywRmdjDefault.getBuilder(); //创建人
        Integer showOrder = dataCrkywRmdjDefault.getShowOrder(); //序号

        bizflowtrail.setBillNo(bizNo);
        bizflowtrail.setUnitID(unitId);
        bizflowtrail.setTrailOrder(trailOrder);
        bizflowtrail.setCurBizNodeID(curBizNodeID);
        bizflowtrail.setNextBizNodeID(nextBizNodeID);
        bizflowtrail.setBizNodeId(bizNodeId);
        bizflowtrail.setBuilder(builder);
        bizflowtrail.setBuildDate(new Date());
        bizflowtrail.setShowOrder(showOrder);
        bizflowtrail.setFlowID(flowId);
        bizflowtrail.setStatus(0);

        return bizflowtrailService.save(bizflowtrail);
    }

}
