package com.unis.crk.cplbusiness.controller;

import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unis.crk.cplbusiness.entity.DataCrkywGysDefault;
import com.unis.crk.cplbusiness.entity.vo.DataCrkywGysDefaultVo;
import com.unis.crk.cplbusiness.service.IDataCrkywGysDefaultService;
import com.unis.crk.utils.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 供应商 前端控制器
 * </p>
 *
 * @author ghh
 * @since 2022-09-14
 */
@Api(value = "供应商",tags = {"供应商"})
@RestController
@RequestMapping("dataCrkywGysDefault")
public class DataCrkywGysDefaultController {

    private final static Logger LOGGER = LoggerFactory.getLogger(DataCrkywGysDefaultController.class);


    @Autowired
    private IDataCrkywGysDefaultService dataCrkywGysDefaultService;


    /**
     * 根据ID查询数据
     * @param id
     * @return
     */
    @ApiOperation(value="查询", notes="根据ID查询数据")
    @GetMapping("getOne/{id}")
    public ResultUtil getOne(@PathVariable("id") String id){
        LOGGER.info("入参为:{}",id);
        ResultUtil<DataCrkywGysDefault> data = new ResultUtil<>();
        if (StringUtils.isEmpty(id)){
            data.failed("传入的主键为空");
            return data;
        }
        data.success(this.dataCrkywGysDefaultService.getById(id));
        return data;
    }

    /**
         * 分页高级查询
         * @return
         */
    @ApiOperation(value="查询", notes="分页高级查询")
    @PostMapping(value="/findPage")
    public ResultUtil findPage(@RequestBody DataCrkywGysDefaultVo pageQuery) {
        ResultUtil<IPage<Map<String,Object>>> data = new ResultUtil<>();
        data.success(this.dataCrkywGysDefaultService.findPageVo(pageQuery));
        return data;
    }
    /**
     * 新增一条数据
     * @param entity
     * @return
     */
    @ApiOperation(value="新增", notes="新增一条数据")
    @PostMapping("add")
    public ResultUtil add(@RequestBody DataCrkywGysDefault entity){
        LOGGER.info("入参为:{}", entity);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (ObjectUtils.isEmpty(entity)){
            data.failed("所接收对象为null");
            return data;
        }
        JSONObject ret  = this.dataCrkywGysDefaultService.add(entity);
        if (ret.getBool("success")){
        data.success(ret.getStr("msg"));
        }else {
        data.failed(ret.getStr("msg"));
        }

        return data;
    }


    /**
     * 修改一条数据
     * @param entity
     * @return
     */
    @ApiOperation(value="修改", notes="修改一条数据")
    @PatchMapping("edit")
    public ResultUtil edit(@RequestBody  DataCrkywGysDefault entity){
        LOGGER.info("入参为:{}", entity);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (ObjectUtils.isEmpty(entity)){
        data.failed("所接收对象为null");
        return data;
        }
        JSONObject ret  = this.dataCrkywGysDefaultService.update(entity);
        if (ret.getBool("success")){
        data.success(ret.getStr("msg"));
        }else {
        data.failed(ret.getStr("msg"));
        }

        return data;
    }


    /**
     * 根据ID删除数据
     * @param id
     * @return
     */
    @ApiOperation(value="删除", notes="根据ID删除数据")
    @DeleteMapping("delete/{id}")
    public ResultUtil delete(@PathVariable("id") String id){
        LOGGER.info("入参为:{}", id);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (StringUtils.isEmpty(id)){
            data.failed("所接收id为null");
            return data;
        }
        JSONObject ret  = this.dataCrkywGysDefaultService.delete(id);
        if (ret.getBool("success")){
        data.success(ret.getStr("msg"));
        }else {
        data.failed(ret.getStr("msg"));
        }
        return data;
    }
    /**
     *获取供应商下拉
     * @param unitId
     * @return
     */
    @ApiOperation(value="查询", notes="通过单位Id查询质检机构拉列表")
    @GetMapping({"/getGysList/{unitId}"})
    public ResultUtil getGysList(@PathVariable(value = "unitId",required = false) String unitId) {
        ResultUtil<List<Map<String,Object>>> data = new ResultUtil<>();
        data.success(this.dataCrkywGysDefaultService.getGysList(unitId));
        return data;
    }

}

