package com.unis.crk.cplbusiness.controller;

import cn.hutool.core.util.IdUtil;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unis.crk.basic.entity.Attaches;
import com.unis.crk.basic.entity.BasicEnum;
import com.unis.crk.basic.service.IAttachesService;
import com.unis.crk.cplbusiness.entity.DispatchEhicleManagement;
import com.unis.crk.cplbusiness.entity.vo.DispatchEhicleManagementVo;
import com.unis.crk.cplbusiness.service.IDispatchEhicleManagementService;
import com.unis.crk.utils.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Iterator;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 调度管理(车辆管理表) 前端控制器
 * </p>
 *
 * @author TangSong
 * @since 2022-05-23
 */
@CrossOrigin
@Api(value = "调度管理(车辆管理表)",tags = {"调度管理(车辆管理表)"})
@RestController
@RequestMapping("dispatchEhicleManagement")
public class DispatchEhicleManagementController {

    private final static Logger LOGGER = LoggerFactory.getLogger(DispatchEhicleManagementController.class);


    @Autowired
    private IDispatchEhicleManagementService dispatchEhicleManagementService;

    @Autowired
    private IAttachesService attachesService;
    /**
     * 根据ID查询数据
     * @param id
     * @return
     */
    @ApiOperation(value="查询", notes="根据ID查询数据")
    @GetMapping("getOne/{id}")
    public ResultUtil getOne(@PathVariable("id") String id){
        LOGGER.info("入参为:{}",id);
        ResultUtil<DispatchEhicleManagement> data = new ResultUtil<>();
        if (StringUtils.isEmpty(id)){
            data.failed("传入的主键为空");
            return data;
        }
        data.success(this.dispatchEhicleManagementService.getOneById(id));
        return data;
    }

    /**
         * 分页高级查询
         * @return
         */
    @ApiOperation(value="查询", notes="分页高级查询")
    @PostMapping(value="/findPage")
    public ResultUtil findPage(@RequestBody DispatchEhicleManagementVo pageQuery) {
        ResultUtil<IPage<Map<String,Object>>> data = new ResultUtil<>();
        data.success(this.dispatchEhicleManagementService.findPageVo(pageQuery));
        return data;
    }

    /**
     * 新增一条数据
     * @param param
     * @return
     */
    @ApiOperation(value="新增", notes="新增一条数据")
    @PostMapping("add")
    public ResultUtil add(@RequestBody DispatchEhicleManagement param){
        LOGGER.info("入参为:{}", param);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (ObjectUtils.isEmpty(param)){
            data.failed("所接收对象为null");
            return data;
        }
        String licensePlateNumber = param.getLicensePlateNumber();
        //车牌号不能重
        QueryWrapper<DispatchEhicleManagement> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("license_plate_number",licensePlateNumber);
        List<DispatchEhicleManagement> dispatchEhicleManagements = dispatchEhicleManagementService.list(queryWrapper);
        if (dispatchEhicleManagements.size() > 0){
            data.failed("车牌号已存在");
            return data;
        }
        String s = IdUtil.simpleUUID();
        param.setDataId(s);
        List<Attaches> attaches = param.getAttaches();
        if(!ObjectUtils.isEmpty(attaches)){
        Iterator<Attaches> iterator = attaches.iterator();
        while (iterator.hasNext()){
            Attaches next = iterator.next();
            next.setReferencesID(s);
        }
            attachesService.saveBatch(param.getAttaches());
        }
        data.success( this.dispatchEhicleManagementService.save(param));
        return  data;
    }

    /**
     * 根据单位id获取下拉
     * @param unitid 节点id
     * @return
     */
    @RequestMapping(value="/getDrop", method=RequestMethod.GET)
    @ApiOperation(value="根据单位id获取下拉车辆", notes="根据单位id获取下拉车辆")
    @ApiImplicitParams({
            @ApiImplicitParam(name="unitid", value="单位id", paramType="query"),
    })

    public List<DispatchEhicleManagement> getDrop(String  unitid) {
        List<DispatchEhicleManagement> list  = dispatchEhicleManagementService.queryDrop(unitid);
        return list;
    }
    /**
     * 修改一条数据
     * @param param
     * @return
     */
    @ApiOperation(value="修改", notes="修改一条数据")
    @PatchMapping("edit")
    public ResultUtil edit(@RequestBody  DispatchEhicleManagement param){
        LOGGER.info("入参为:{}", param);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (ObjectUtils.isEmpty(param)){
        data.failed("所接收对象为null");
        return data;
        }
        JSONObject ret  = this.dispatchEhicleManagementService.update(param);
        if(!ObjectUtils.isEmpty(param.getAttaches())){
            attachesService.saveOrUpdateBatch(param.getAttaches());
        }
        if (ret.getBool("success")){
        data.success(ret.getStr("msg"));
        }else {
        data.failed(ret.getStr("msg"));
        }

        return data;
    }


    /**
     * 根据ID删除数据
     * @param id
     * @return
     */
    @ApiOperation(value="删除", notes="根据ID删除数据")
    @DeleteMapping("delete/{id}")
    public ResultUtil delete(@PathVariable("id") String id){
        LOGGER.info("入参为:{}", id);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (StringUtils.isEmpty(id)){
            data.failed("所接收id为null");
            return data;
        }
        JSONObject ret  = this.dispatchEhicleManagementService.delete(id);
        int i = attachesService.deletAttachesByReferencesID(id);
        if (ret.getBool("success")){
        data.success(ret.getStr("msg"));
        }else {
        data.failed(ret.getStr("msg"));
        }
        return data;
    }


}

