package com.unis.crk.violation.controller;

import com.unis.crk.violation.entity.ViolationWoring;
import com.unis.crk.violation.entity.vo.ViolationWoringVo;
import com.unis.crk.violation.service.IViolationWoringService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.unis.crk.utils.ResultUtil;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import java.util.Map;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import cn.hutool.json.JSONObject;


/**
 * <p>
 * 表E.34 违规预警信息 前端控制器
 * </p>
 *
 * @author Wys
 * @since 2023-06-15
 */
@CrossOrigin
@Api(value = "表E.34 违规预警信息",tags = {"表E.34 违规预警信息"})
@RestController
@RequestMapping("violationWoring")
public class ViolationWoringController {

    private final static Logger LOGGER = LoggerFactory.getLogger(ViolationWoringController.class);


    @Autowired
    private IViolationWoringService violationWoringService;


    /**
     * 根据ID查询数据
     * @param id
     * @return
     */
    @ApiOperation(value="查询", notes="根据ID查询数据")
    @GetMapping("getOne/{id}")
    public ResultUtil getOne(@PathVariable("id") String id){
        LOGGER.info("入参为:{}",id);
        ResultUtil<ViolationWoring> data = new ResultUtil<>();
        if (StringUtils.isEmpty(id)){
            data.failed("传入的主键为空");
            return data;
        }
        data.success(this.violationWoringService.getById(id));
        return data;
    }

    /**
         * 分页高级查询
         * @return
         */
    @ApiOperation(value="查询", notes="分页高级查询")
    @PostMapping(value="/findPage")
    public ResultUtil findPage(@RequestBody ViolationWoringVo pageQuery) {
        ResultUtil<IPage<Map<String,Object>>> data = new ResultUtil<>();
        data.success(this.violationWoringService.findPageVo(pageQuery));
        return data;
    }

    /**
     * 新增一条数据
     * @param item
     * @return
     */
    @ApiOperation(value="新增", notes="新增一条数据")
    @PostMapping("add/{unitid}")
    public ResultUtil add(@PathVariable("unitid") String unitid,@RequestBody ViolationWoringVo item){
        LOGGER.info("入参为:{}", item);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (StringUtils.isEmpty(unitid)){
            data.failed("所接收unitid为null");
            return data;
        }
        if (ObjectUtils.isEmpty(item)){
            data.failed("所接收对象为null");
            return data;
        }
        JSONObject ret  = this.violationWoringService.saveData(unitid,item);
        if (ret.getBool("success")){
            data.success(ret.getStr("msg"));
        }else {
            data.failed(ret.getStr("msg"));
        }
        return data;
    }


    /**
     * 修改一条数据
     * @param item
     * @return
     */
    @ApiOperation(value="修改", notes="修改一条数据")
    @PatchMapping("edit")
    public ResultUtil edit(@RequestBody  ViolationWoring item){
        LOGGER.info("入参为:{}", item);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (ObjectUtils.isEmpty(item)){
        data.failed("所接收对象为null");
        return data;
        }
        JSONObject ret  = this.violationWoringService.update(item);
        if (ret.getBool("success")){
        data.success(ret.getStr("msg"));
        }else {
        data.failed(ret.getStr("msg"));
        }

        return data;
    }


    /**
     * 根据ID删除数据
     * @param id
     * @return
     */
    @ApiOperation(value="删除", notes="根据ID删除数据")
    @DeleteMapping("delete/{id}")
    public ResultUtil delete(@PathVariable("id") String id){
        LOGGER.info("入参为:{}", id);
        ResultUtil<Boolean> data = new ResultUtil<>();
        if (StringUtils.isEmpty(id)){
            data.failed("所接收id为null");
            return data;
        }
        JSONObject ret  = this.violationWoringService.delete(id);
        if (ret.getBool("success")){
        data.success(ret.getStr("msg"));
        }else {
        data.failed(ret.getStr("msg"));
        }
        return data;
    }


}

